"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = __importDefault(require("@pnpm/logger"));
const path = require("path");
const isInnerLink = require("is-inner-link");
const isSubdir = require("is-subdir");
const fs = require("mz/fs");
async function safeIsInnerLink(projectModulesDir, depName, opts) {
    try {
        const link = await isInnerLink(projectModulesDir, depName);
        if (link.isInner)
            return true;
        if (isSubdir(opts.virtualStoreDir, link.target) || isSubdir(opts.storeDir, link.target))
            return true;
        return link.target;
    }
    catch (err) {
        if (err.code === 'ENOENT')
            return true;
        if (opts.hideAlienModules) {
            logger_1.default.warn({
                message: `Moving ${depName} that was installed by a different package manager to "node_modules/.ignored`,
                prefix: opts.projectDir,
            });
            const ignoredDir = path.join(projectModulesDir, '.ignored', depName);
            await fs.mkdir(path.dirname(ignoredDir), { recursive: true });
            await fs.rename(path.join(projectModulesDir, depName), ignoredDir);
        }
        return true;
    }
}
exports.default = safeIsInnerLink;
//# sourceMappingURL=safeIsInnerLink.js.map